(*
   P R V 2 4 . P A S
   V24-Routinen nach DL1BFF und DK2UI
   angepasst/erweitert DL1BHO, im Juni 1988
*)


unit PRV24;


interface

uses dos,
     crt,
     PRDefs,
     PRLib;


CONST  MaxBuf       = 1024;

TYPE   ComType      = RECORD
                        Base        : INTEGER;
                        V24Buffer   : ARRAY[1..MaxBuf] OF BYTE;
                        BufferPos   : INTEGER;
                        UserPos     : INTEGER;
                        IntNummer   : INTEGER;
                        Old_Vector  : POINTER;
                        Old_IER     : BYTE;  (* Old InterruptEnableRegister *)
                      END;


VAR    Com          : ARRAY[1..2] OF ComType;
       V24Active    : BOOLEAN;
       AktivCom     : INTEGER;
       ComPortBase  : ARRAY[1..4] OF INTEGER ABSOLUTE $0:$0400;
                      (* Adressen der ComPorts stehen im Ram *)
       Old_IntMask  : BYTE;


  procedure setV24(Baud : integer);
  function  Zeichen : boolean;
  procedure writeaux(Zeile : str255);
  function  ReSync:BOOLEAN;
  procedure readaux(var ch232 : char);
  PROCEDURE V24_Close;
  procedure switch_V24(RS232_Nummer : integer);


implementation


VAR    i   : INTEGER;


(*$F+*)
PROCEDURE Com1_Int;
(*$F-*)
interrupt;
BEGIN
  WITH Com[1] DO BEGIN
    V24Buffer[BufferPos] := Port[Base];
    IF BufferPos >= MaxBuf THEN BufferPos := 1 ELSE Inc(BufferPos);
  END;
  Port[$20]:=$20;
End;


(*$F+*)
PROCEDURE Com2_Int;
(*$F-*)
interrupt;
BEGIN
  WITH Com[2] DO BEGIN
    V24Buffer[BufferPos] := Port[Base];
    IF BufferPos >= MaxBuf THEN BufferPos := 1 ELSE Inc(BufferPos);
  END;
  Port[$20]:=$20;
End;


PROCEDURE V24_Close;
BEGIN
  IF V24Active THEN BEGIN
    IF Hardware = 0 THEN BEGIN
      Port[Com[1].Base+1] := Com[1].Old_IER;
      Port[Com[2].Base+1] := Com[2].Old_IER;
      SetIntVec(12,Com[1].Old_Vector);
      SetIntVec(11,Com[2].Old_Vector);
    END
    ELSE
    BEGIN
      Port[Com[Hardware].Base+1] := Com[Hardware].Old_IER;
      SetIntVec(Com[Hardware].IntNummer,Com[Hardware].Old_Vector);
    END;
    Port[$21] := Old_IntMask;
    V24Active := FALSE;
  END;
END;


PROCEDURE V24_Init(ComNummer : INTEGER ; Baud : INTEGER);
VAR   b   : INTEGER;
BEGIN
  b := Round(115200 / Baud);
  WITH Com[ComNummer] DO BEGIN
    IF (Base = 0) THEN BEGIN
      (* Schnittstelle nicht installiert ! *)
      WRITELN(^M^G'Schnittstelle fr COM',ComNummer,': nicht installiert !!!');
      Halt;
    END;
    port[Base+3]:=$80;       (* LCR : DLAB=1 *)
    port[Base+0]:=Lo(b);     (* $06 = 19200 bd, $0C = 9600, $18 = 4800 *)
    port[Base+1]:=Hi(b);     (* HI Baud *)
    port[Base+3]:=$03;       (* LCR NoParity 8Data 1Stop : DLAB=0 *)
    port[Base+4]:=$0B;       (* MCR rts und dtr = H *)
    Old_IER := Port[Base+1]; (* Alten Interruptstatus speichern *)
    port[Base+1]:=$01;       (* Interrupt bei Empfangsdaten *)
    BufferPos := 1;
    UserPos := 1;

    if V24Active THEN exit;

    getintvec(IntNummer,Old_Vector);
    IF ComNummer = 1 THEN setintvec(12,@Com1_Int) ELSE SetIntVec(11,@Com2_Int);

  END;
END;


procedure setV24;
var k    : integer;
Begin
  Inline($FA);   (* cli *)
  if Hardware = 0 then
  Begin
    (* COM1 und COM2 sollen installiert werden *)
    for k := 1 to 2 do
    Begin
      IF Schnitte[k].TNC THEN V24_Init(k,Baud);
    End;
  End
  else
  Begin
    (* COM1 oder COM2 soll installiert werden *)
    V24_Init(Hardware,Baud);
    AktivCom := Hardware;
  End;
  Old_IntMask := Port[$21];
  Port[$21] := Port[$21] AND $E7;   (* Interrupt-Maske *)
  V24Active := TRUE;
  Inline($FB);   (* sti *)
End;


FUNCTION Zeichen;
BEGIN
  WITH Com[AktivCom] DO
  BEGIN
    Zeichen := (BufferPos <> UserPos);
  END;
END;


PROCEDURE WriteAux;
VAR   i  : INTEGER;
BEGIN
  WITH Com[AktivCom] DO BEGIN
    FOR i := 1 TO Length(Zeile) DO BEGIN
      REPEAT UNTIL (Port[Base+5] AND 32) = 32;
      Port[Base] := Ord(Zeile[i]);
    END;
  END;
END;


FUNCTION ReadV24 : CHAR;
BEGIN
  WITH Com[AktivCom] DO BEGIN
    Inline($FA);   (* cli (bloss jetzt keine Interrupts!) *)
    ReadV24 := Chr(V24Buffer[UserPos]);
    IF UserPos >= MaxBuf THEN UserPos := 1 ELSE Inc(UserPos);
    Inline($FB);   (* sti *)
  END;
END;


{----------------------------------------------------------------------------+
|                                                                            |
| Hostmode Synchronisation wiederherstellen (Nach KB5MU Hostmode Guide)      |
|                                                                            |
| bernommen aus PCT von Chris, DD6CV                                        |
|                                                                            |
+----------------------------------------------------------------------------}
FUNCTION ReSync;
VAR
  i       : INTEGER;
  ch1     : CHAR;

BEGIN
  Inc(Resync_Z); beep(1000,100);
  Fenster(25,7,28,8,'RESYNCHRONISATION');
  gotoxy(27,10);  write('Hostmode-Synchr. gestrt!');
  gotoxy(27,11);  write('Re-Synchr. wird versucht.');
  gotoxy(27,13);  WRITE('>>> Versuch Nr.     ');

  FOR i:=1 TO 1000 DO                  { Eventuelle im TNC TX Buffer be-     }
    WHILE Zeichen DO ch1 := ReadV24;   { findliche Zeichen auslesen.         }

  i:=0;
  REPEAT
    writeaux(^A);                      { CTRL-A schicken                     }
    WRITE(^H^H^H^H,i:4);               { Anzahl der Versuche anzeigen        }
    Delay(20);                         { ca. 100 mSek warten                 }
    if i > 80 then delay((i-80)*10);   { auf 9600 Baud optimiert             }
    Inc(i);
  UNTIL (i = 100  ) OR Zeichen;         { 256 ^A oder bis TNC antwortet      }

  IF Zeichen THEN                      { TNC hat geantwortet, also noch im   }
  BEGIN                                { Hostmode                            }
    FOR i:=1 TO 1000 DO                { TNC TX Buffer leeren                }
      WHILE Zeichen DO ch1 := ReadV24;
    ReSync:=TRUE;
  END
  ELSE                                 { TNC hat nicht geantwortet, also     }
    ReSync:=FALSE;                     { nicht (mehr) im Hostmode            }
  Neu_Bild;                            (* Bildschirm wieder aufbauen *)
END;


procedure readaux;
var
  TimeOut   : LongInt;
  Sync      : boolean;
  Kanal     : integer;

  function gettimer: longint;
  begin
    inline($FA);                               {cli}
     GetTimer:=meml[$0040:$006c];
    inline($FB);                               {sti}
  end;

Begin
  Sync := true;
  if not Zeichen then
  Begin
    TimeOut := GetTimer; inc(timeout,54);           (* 3 Sekunden Timeout *)
    REPEAT if timeout-gettimer >54  then timeout:=timeout-1573039;
    UNTIL (Zeichen OR (GetTimer >= TimeOut));
    if not zeichen THEN BEGIN            (* TNC hat nicht geantwortet ... *)
      Sync := false;
      IF NOT ReSync then
      Begin                                 (* ReSync nicht erfolgreich ! *)
        clrscr; Cursor_ein;
        writeln(^M'Re-Synchronisation war nicht erfolgreich! Programm beendet!');
        for Kanal := 0 to maxLink do
        Begin (* Files fr alle Kanle schlieen, damit nichts verlorengeht *)
          if K[Kanal]^.Save then
          Begin
            close(K[Kanal]^.RXFile);
            K[Kanal]^.Save := false;
          End;
        End;
        V24_Close;
        HALT(0);
      End;
    End;
  End;
  if Sync then  ch232 := ReadV24                       (* Zeichen einlesen *)
          else  ch232 := #0;
End;


(*
   V24-Schalter, wenn an einer Schnittstelle mehrere TNCs betrieben werden
   oder wenn 2 Schnittstellen in Betrieb sind.
*)

procedure switch_V24;
Begin
  if Hardware = 0 then
  Begin                                       (* COM1 und COM2 installiert *)
    if active_Port <> RS232_Nummer then
    Begin
      if (RS232_Nummer >=1) and (RS232_Nummer <= 2) then BEGIN
        active_Port := RS232_Nummer;
        AktivCom := RS232_Nummer;
      END
      else
      Begin
        writeln(^G^G'Falsche TNC-Konfiguration im File CONFIG.PR !!!');
        halt(0);
      End;
    End;
  End
  else
  Begin             (* nur eine physikalische Schnittstelle konfiguriert *)
    if active_Port <> RS232_Nummer then WITH Com[AktivCom] DO
    Begin
      active_Port := RS232_Nummer;
      case RS232_Nummer of
       1 : port[Base + 4] := ($08 + $03);
       2 : port[Base + 4] := ($08 + $02);
       3 : port[Base + 4] := ($08 + $01);
       4 : port[Base + 4] := ($08 + $00);
       else
         Begin
           writeln(^G^G'Falsche TNC-Konfiguration im File CONFIG.PR !!!');
           V24_Close; halt(0);
         End;
      end;
      delay(10);
    End;
  End;
End;


BEGIN
  V24Active := FALSE;
  FOR i := 1 TO 2 DO Com[i].Base := ComPortBase[i];
  Com[1].IntNummer := 12;
  Com[2].IntNummer := 11;
END.
